@@ -1,8 +1,8 @@
-# encoding: utf-8
 """
 Models for django-sphinxdoc.
 
 """
+from django.conf import settings
 from django.db import models
 from django.utils.translation import ugettext_lazy as _
 
@@ -17,10 +17,10 @@ class Project(models.Model):
     """
     name = models.CharField(max_length=100)
     slug = models.SlugField(unique=True,
-                            help_text=_(u'Used in the URL for the project. '
+                            help_text=_('Used in the URL for the project. '
                                         'Must be unique.'))
     path = models.CharField(max_length=255, validators=[validate_isdir],
-                            help_text=_(u'Directory that contains Sphinx’ '
+                            help_text=_('Directory that contains Sphinx’ '
                                         '<tt>conf.py</tt>.'))
 
     class Meta:
@@ -30,6 +30,16 @@ class Project(models.Model):
     def __unicode__(self):
         return self.name
 
+    def is_allowed(self, user):
+        protected = getattr(settings, 'SPHINXDOC_PROTECTED_PROJECTS', {})
+        if not self.slug in protected:
+            # Project not protected, publicly visible
+            return True
+        if (not user.is_authenticated() or
+            not user.has_perms(protected[self.slug])):
+            return False
+        return True
+
     @models.permalink
     def get_absolute_url(self):
         return ('doc-index', (), {'slug': self.slug})
